--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

local dewdrop = AceLibrary("Dewdrop-2.0")
local quixote = AceLibrary("Quixote-1.0")
local crayon = AceLibrary("Crayon-2.0")
local tablet = AceLibrary("Tablet-2.0")

function nQuestLog:UpdateEvents()
	-- Zone change events
	if (self.db.profile.showQuestHighlight and not self.ZONEHOOKED) then
		self:RegisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
		self.ZONEHOOKED = true
		self:OnZoneChange()
	elseif(not self.db.profile.showQuestHighlight and self.ZONEHOOKED) then
		self:UnregisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
		self.ZONEHOOKED = false
	end
	if (not self.db.profile.showQuestHighlight) then
		self.highlightList = {}
	end
	
	-- Tooltip events:
	if (self.db.profile.showToolTip and not self.TOOLTIPHOOKED) then
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT","OnMouseoverUnit")
		self:HookScript(GameTooltip, "OnTooltipSetItem")
		self.TOOLTIPHOOKED = true
	elseif (not self.db.profile.showToolTip and self.TOOLTIPHOOKED) then
		self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT","OnMouseoverUnit")
		self:Unhook(GameTooltip, "OnTooltipSetItem")
		self.TOOLTIPHOOKED = false
	end
end

-------------------------------------------------- Zone Events : ------------------------------------------------

function nQuestLog:OnZoneChange(quick)
	self.zonerefresh = false
	self.highlightList = {}
	if (not self.db.profile.showQuestHighlight) then
		return
	end
	local strZoneText = string.lower(GetZoneText())
	local strSubZoneText = string.lower(GetSubZoneText())

	for _,iQuestId in quixote:QuestsByLevel() do
		local strTitle, _, _, strObjective, _, _, strZone = quixote:GetQuestById(iQuestId)
		-- Check for zone text in short quest objective description
		if strSubZoneText ~= "" and string.find(string.lower(strObjective),strSubZoneText,1,true) then
			self.highlightList[strTitle] = true
		end
		
		-- Now check the quest description against the main zone, maybe we got out of zone quests.
		if strZoneText ~= "" and string.lower(strZone) ~= strZoneText and string.find(string.lower(strObjective),strZoneText,1,true) then
			self.highlightList[strTitle] = true
		end
	end
	
	if (not quick) then
		self:Refresh()
	end
end

-------------------------------------------------- UI Events : --------------------------------------------------

function nQuestLog:OnTooltipUpdate()
	local cat = tablet:AddCategory()
	
	local strStatus = "Visible"
	if (not nQuestLogFrame:IsVisible()) then
		strStatus = "Hidden"
		if (self.db.char["visible"]) then
			strStatus = "Hidden (no quests to track)"
		end
	end
	
	cat:AddLine("text","Tracker " .. strStatus)
	
	tablet:SetHint("\n|cffeda55fClick|r to toggle Quest Tracker\n|cffeda55fRight Click|r for options")
end

function nQuestLog:OnClick(button)
	-- FuBar OnClick
	if (nQuestLogFrame:IsVisible()) then
		self:ToggleVisible(false)
	else
		self:ToggleVisible(true)
	end
end

function nQuestLog:OnTextUpdate()
	local iQuests = quixote:GetNumQuests()
	local color = crayon:GetThresholdHexColor((MAX_QUESTS - iQuests) / MAX_QUESTS)
	self:SetText(crayon:Colorize(color, iQuests .. "/" .. MAX_QUESTS))
end

function nQuestLog:OnMenuRequest()
	self:RefreshMenu()
	dewdrop:FeedAceOptionsTable(self.mainMenu)
end

function nQuestLog:OnQuestLogRightClick(strZone)
	if (strZone == nil) then
		return
	end
	
	local strZoneVisibility = "Hide Zone"
	if self.db.char.hiddenZones[strZone] then
		strZoneVisibility = "Show Zone"
	end
	local color = GetColor(self.db.profile.zoneHeader)
	strZoneVisibility = crayon:Colorize(color,strZoneVisibility)
	
	local popupMenu = {
		type='group',
		args = {
	         showHide = {
	            type = 'execute',
	            name = strZoneVisibility,
	            desc = 'Toggle zone visibility',
	            order = 1,
	            func = function()
	            	if (self.db.char.hiddenZones[strZone]) then
	            		self.db.char.hiddenZones[strZone] = nil
	            	else
	            		self.db.char.hiddenZones[strZone] = true
	            	end
	            	self:Refresh()
	            	dewdrop:Close()
	            end
	        },
		}
	}
	
	for _,iQuestId in quixote:QuestsByZone(strZone) do
		local strTitle, iLevel, strTag, strObjective, iComplete, iObjectives, strZone = quixote:GetQuestById(iQuestId)
		local _, iTotalGot, iTotalNeeded = self:Format_QuestObjectives(strTitle,iObjectives,strObjective,iQuestId)
		local strFancyTitle = self:Format_QuestTitle(strTitle,iLevel,strTag,iTotalGot,iTotalNeeded,iComplete)
		
		popupMenu.args[strTitle] = {
			order = iQuestId + 5,
			name = strFancyTitle,
			type = "toggle",
			desc = "Toggle showing " .. strTitle,
			get = function()
				return self.db.char.watchedQuests[strTitle]
			end,
			set = function(v)
				self.db.char.watchedQuests[strTitle] = nil
				if (v) then
					self.db.char.watchedQuests[strTitle] = true
				end
				self:Refresh()
			end
		}
	end
	
	dewdrop:Open( nQuestLogFrame,
		'children', function() dewdrop:FeedAceOptionsTable(popupMenu ) end,
		'cursorX', true, 'cursorY', true )
end

function nQuestLog:OnQuestLogClick(questid,zone)
	-- Deal with zone
	if (questid == nil) then
		if self.db.char.hiddenZones[zone] then
			self.db.char.hiddenZones[zone] = nil
		else
			self.db.char.hiddenZones[zone] = true
		end
		self:Refresh()
		return
	end

	-- Deal with quest
	local title,level,_,_,_,objectives = quixote:GetQuestById(questid)
	if (title == nil) then
		return
	end
	
	if (IsShiftKeyDown()) then
		-- Link quest to chat
		local text = "[" .. level .. "]" .. title .. " "

		if (IsControlKeyDown()) then
			-- Control key is down, link total objectives as well
			local totalGot,totalNeeded = 0,0
			for i=1,objectives do
				local _, _, got, needed = quixote:GetQuestStatusById(questid, i)
				totalGot = totalGot + got
				totalNeeded = totalNeeded + needed
			end
			text = text .. totalGot .. "/" .. totalNeeded .. " "
		end
		
		if (ChatFrameEditBox:IsVisible()) then
			ChatFrameEditBox:Insert(text)
		else
			local chatType = DEFAULT_CHAT_FRAME.editBox:GetAttribute("chatType")
			SendChatMessage(text, chatType)
		end
	else
		if self:IsModuleEnabled("DetailsFrame") then
			self:GetModule("DetailsFrame"):ShowLog(questid)
		else
			local wasHidden = not QuestLogFrame:IsVisible()
			QuestLogFrame:Hide()
			
			if (GetQuestLogSelection() ~= questid or wasHidden) then
				SelectQuestLogEntry(questid)
				QuestLogFrame:Show()
			end
		end
	end
end

-------------------------------------------------- Tooltip Events : --------------------------------------------------

function nQuestLog:OnMouseoverUnit()
	local strName,iGot,iNeeded = quixote:IsQuestMob(UnitName('mouseover'))
	if strName then
		GameTooltip:AddDoubleLine(strName, iGot..'/'..iNeeded, 1, 1, 1, crayon:GetThresholdColor(iGot/iNeeded))
		GameTooltip:Show()
	end
end

function nQuestLog:OnTooltipSetItem(tooltip, ...)
	local strName,iGot,iNeeded = quixote:IsQuestItem(tooltip:GetItem())
	if (strName) then
		tooltip:AddDoubleLine(strName, iGot..'/'..iNeeded, 1, 1, 1, crayon:GetThresholdColor(iGot/iNeeded))
		tooltip:Show()
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...)
end



-------------------------------------------------- Quest Events: --------------------------------------------------

function nQuestLog:OnRefresh()
	if self.zonerefresh then
		if self.db.profile.showQuestHighlight then
			self:OnZoneChange(true)
		end
		self.refresh = true
	end
	if self.refresh then
		self:Refresh()
	end
end

function nQuestLog:OnQuestLog_Ready()
	self:OnZoneChange(true)
	if self.db.char["visible"] then
		self:ToggleVisible(true)
	end
	self:UpdateText()
end

function nQuestLog:OnQuest_Gained(strTitle)
	if (self.db.profile.trackNewQuests) then
	
		-- For smart quest tracking:
		if (self.db.profile.smartObjectiveTracking and not self.db.profile.hideAllObjectives) then
			-- If we're using smart quest tracking, and not hiding all objectives per default, pretend we got a quest item when we pickup the quest.
			local _, _, _, _, _, _, strZone = quixote:GetQuestByName(strTitle)
			if (not self.db.char.hiddenZones[strZone]) then
				-- but only when the zone isn't hidden
				self.ObjectivesDB[strTitle] = GetTime()
			end
		end
		
		self.db.char.watchedQuests[strTitle] = true
	end
	self:UpdateText()
	
	-- Sometimes quest objectives descriptions are missing until next refresh
	self.refresh = true
	self.zonerefresh = true
	self:ScheduleEvent(self.Refresh, 2, self)
end

function nQuestLog:OnQuest_Lost(strTitle)
	self.db.char.watchedQuests[strTitle] = nil
	self.ObjectivesDB[strTitle] = nil
	self.refresh = true
	self:UpdateText()
end

function nQuestLog:OnQuest_Leaderboard_Update(strTitle, _, _, strDesc, iHad, iGot, iNeeded)
	-- For smart quest tracking:
	if (self.db.profile.smartObjectiveTracking and self.ObjectivesDB[strTitle] ~= true and self.ObjectivesDB[strTitle] ~= false) then
		self.ObjectivesDB[strTitle] = GetTime()
	end
	
	if (self.db.profile.trackWhenLooting and not self.db.char.watchedQuests[strTitle]) then
		self.db.char.watchedQuests[strTitle] = true
		self:Print("Found " .. strDesc .. " for " .. strTitle .. " adding to quest tracker.")
	end
	
	if (self.db.profile.alertFinishedQuest) then
		local _, _, _, _, complete = quixote:GetQuestByName(strTitle)
		if complete == 1 then
			self:Print("Finished quest " .. strTitle)
			self.refresh = true
			return
		end
	end

	if (iGot == iNeeded and self.db.profile.alertObjectiveQuest) then
		self:Print("Finished objective " .. strDesc)
	else
		if (self.db.profile.alertLootedQuestItem and not self.db.profile.alertLootedQuestItemHiddenOnly) then
			-- Show alert, not locked to hidden only
			self:Print(strDesc .. " " .. iGot .. "/" ..iNeeded)
		elseif(self.db.profile.alertLootedQuestItemHiddenOnly and not self.db.char.watchedQuests[strTitle]) then
			-- Show is hidden only, but quest is hidden.
			self:Print(strDesc .. " " .. iGot .. "/" ..iNeeded)
		end
	end
	
	self.refresh = true
end
